#!/usr/bin/env python

import re
import random
from operator import mul

"""
Functions for working with Brazilian CEI identifiers.

"""

NONDIGIT = re.compile(r'[^0-9]')
CEI_WEIGHTS = [7, 4, 1, 8, 5, 2, 1, 6, 3, 7, 4]

def clean_cei(cei):
    """Takes a CEI and turns it into a string of only numbers."""
    return NONDIGIT.sub('', str(cei))

def validate_cei(cei):
    """Check whether CEI is valid."""
    cei = clean_cei(cei)
    # all complete CEI are 12 digits long
    if len(cei) != 12:
        return False
    digits = [int(k) for k in cei]  # identifier digits
    return _cei_check(digits[:-1]) == digits[-1]

def cei_check_digit(cei):
    """Find check digit needed to make a CEI valid."""
    cei = clean_cei(cei)
    if len(cei) < 11:
        raise ValueError('CEI must have at least 11 digits: {0}'.format(cei))
    digits = [int(k) for k in cei[:12]]
    return _cei_check(digits)

def format_cei(cei):
    """Applies typical 00.000.00000/00 formatting to CEI."""
    fmt = '{0}.{1}.{2}/{3}'
    cei = clean_cei(cei)
    return fmt.format(cei[:2], cei[2:5], cei[5:10], cei[10:])

def pad_cei(cei, validate=True):
    """Takes a CEI that probably had leading zeros and pads it."""
    cei = clean_cei(cei)
    cei = '%0.012i' % int(cei)
    if validate and not validate_cei(cei):
        raise ValueError('Invalid CEI: {0}'.format(cei))
    return cei

def random_cei(formatted=True):
    """Create a random, valid CEI identifier."""
    uf = random.randint(11, 53)
    stem = '{0}{1}'.format(uf, random.randint(100000000, 999999999))
    cei = '{0}{1}'.format(stem, cei_check_digit(stem))
    if formatted:
        return format_cei(cei)
    return cei

def _cei_check(digits):
    """Calculate check digit from iterable of integers."""
    digsum = sum(mul(*k) for k in zip(CEI_WEIGHTS, digits))
    modulo = (sum(divmod(digsum % 100, 10)) % 10)
    if modulo == 0:
        return 0
    return 10 - modulo



#!/usr/bin/env python

import re
import random
from operator import mul

"""
Functions for working with Brazilian CPF identifiers.

"""

NONDIGIT = re.compile(r'[^0-9]')
CPF_WEIGHTS  = [1, 2, 3, 4, 5, 6, 7, 8, 9]

def clean_cpf(cpf):
    """Takes a CPF and turns it into a string of only numbers."""
    return NONDIGIT.sub('', str(cpf))

def validate_cpf(cpf):
    """Check whether CPF is valid."""
    cpf = clean_cpf(cpf)
    # all complete CPF are 11 digits long
    if len(cpf) != 11:
        return False
    digits = [int(k) for k in cpf]  # identifier digits
    checks = digits[-2:]  # last two digits are check digits
    # validate the first check digit
    cs = (sum([mul(*k) for k in zip(CPF_WEIGHTS, digits[:-2])]) % 11) % 10
    if cs != checks[0]:
        return False  # first check digit is not correct
    # validate the second check digit
    cs = (sum([mul(*k) for k in zip(CPF_WEIGHTS, digits[1:-1])]) % 11) % 10
    if cs != checks[1]:
        return False  # second check digit is not correct
    # both check digits are correct
    return True

def cpf_check_digits(cpf):
    """Find two check digits needed to make a CPF valid."""
    cpf = clean_cpf(cpf)
    if len(cpf) < 9:
        raise ValueError('CPF must have at least 9 digits: {0}'.format(cpf))
    digits = [int(k) for k in cpf[:10]]
    # find the first check digit
    cs = (sum([mul(*k) for k in zip(CPF_WEIGHTS, digits)]) % 11) % 10
    # find the second check digit
    digits.append(cs)
    return cs, (sum([mul(*k) for k in zip(CPF_WEIGHTS, digits[1:])]) % 11) % 10

def format_cpf(cpf):
    """Applies typical 000.000.000-00 formatting to CPF."""
    fmt = '{0}.{1}.{2}-{3}'
    cpf = clean_cpf(cpf)
    return fmt.format(cpf[:3], cpf[3:6], cpf[6:9], cpf[9:])

def pad_cpf(cpf, validate=True):
    """Takes a CPF that probably had leading zeros and pads it."""
    cpf = clean_cpf(cpf)
    cpf = '%0.011i' % int(cpf)
    if validate and not validate_cpf(cpf):
        raise ValueError('Invalid CPF: {0}'.format(cpf))
    return cpf

def random_cpf(formatted=True):
    """Create a random, valid CPF identifier."""
    stem = random.randint(100000000, 999999999)
    cpf = str(stem) + '{0}{1}'.format(*cpf_check_digits(stem))
    if formatted:
        return format_cpf(cpf)
    return cpf




#!/usr/bin/env python

import re
import random
from operator import mul
from collections import namedtuple

"""
Functions for working with Brazilian company identifiers (CNPJ).

"""

NONDIGIT = re.compile(r'[^0-9]')
CNPJ_FIRST_WEIGHTS  = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
CNPJ_SECOND_WEIGHTS = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
CNPJ = namedtuple('CNPJ', ['cnpj', 'firm', 'establishment', 'check'])


def clean_cnpj(cnpj):
    """Takes a CNPJ and turns it into a string of only numbers."""
    return NONDIGIT.sub('', str(cnpj))

def validate_cnpj(cnpj):
    """Check whether CNPJ is valid."""
    cnpj = clean_cnpj(cnpj)
    # all complete CNPJ are 14 digits long
    if len(cnpj) != 14:
        return False
    checks = [int(k) for k in cnpj[12:]]  # check digits
    digits = [int(k) for k in cnpj[:13]]  # identifier digits
    # validate the first check digit
    cs = sum([mul(*k) for k in zip(CNPJ_FIRST_WEIGHTS, digits[:-1])]) % 11
    cs = 0 if cs < 2 else 11 - cs
    if cs != checks[0]:
        return False  # first check digit is not correct
    # validate the second check digit
    cs = sum([mul(*k) for k in zip(CNPJ_SECOND_WEIGHTS, digits)]) % 11
    cs = 0 if cs < 2 else 11 - cs
    if cs != checks[1]:
        return False  # second check digit is not correct
    # both check digits are correct
    return True

def cnpj_check_digits(cnpj):
    """Find two check digits needed to make a CNPJ valid."""
    cnpj = clean_cnpj(cnpj)
    if len(cnpj) < 12:
        raise ValueError('CNPJ must have at least 12 digits: {0}'.format(cnpj))
    digits = [int(k) for k in cnpj[:13]]
    # find the first check digit
    cs = sum([mul(*k) for k in zip(CNPJ_FIRST_WEIGHTS, digits)]) % 11
    check = 0 if cs < 2 else 11 - cs
    # find the second check digit
    digits.append(check)
    cs = sum([mul(*k) for k in zip(CNPJ_SECOND_WEIGHTS, digits)]) % 11
    if cs < 2:
        return check, 0
    return check, 11 - cs

def cnpj_from_firm_id(firm, establishment='0001'):
    """Takes first 8 digits of a CNPJ (firm identifier) and builds a valid,
       complete CNPJ by appending an establishment identifier and calculating
       necessary check digits.
    """
    cnpj = clean_cnpj('{0}{1}'.format(firm, establishment))
    checks = ''.join([str(k) for k in cnpj_check_digits(cnpj)])
    return cnpj + checks

def format_cnpj(cnpj):
    """Applies typical 00.000.000/0000-00 formatting to CNPJ."""
    fmt = '{0}.{1}.{2}/{3}-{4}'
    cnpj = clean_cnpj(cnpj)
    return fmt.format(cnpj[:2], cnpj[2:5], cnpj[5:8], cnpj[8:12], cnpj[12:])

def pad_cnpj(cnpj, validate=True):
    """Takes a CNPJ that probably had leading zeros and pads it."""
    cnpj = clean_cnpj(cnpj)
    cnpj = '%0.014i' % int(cnpj)
    if validate and not validate_cnpj(cnpj):
        raise ValueError('Invalid CNPJ: {0}'.format(cnpj))
    return cnpj

def parse_cnpj(cnpj, formatted=True, validate=True):
    """Split CNPJ into firm, establishment, and check digit parts."""
    cnpj = pad_cnpj(cnpj, validate)
    estbl, check = cnpj[8:12], cnpj[12:]
    if formatted:
        cnpj = format_cnpj(cnpj)
        firm = cnpj[:10]
        return CNPJ(cnpj, firm, estbl, check)
    else:
        firm = cnpj[:8]
        check = tuple(int(k) for k in check)
        return CNPJ(int(cnpj), int(firm), int(estbl), check)

def random_cnpj(formatted=True):
    """Create a random, valid CNPJ identifier."""
    firm = random.randint(10000000, 99999999)
    establishment = random.choice(['0001', '0002', '0003', '0004', '0005'])
    cnpj = cnpj_from_firm_id(firm, establishment)
    if formatted:
        return format_cnpj(cnpj)
    return cnpj



#!/usr/bin/env python

import re
from random import randint
from operator import mul

"""
Functions for working with Brazilian PIS/PASEP identifiers.

"""

NONDIGIT = re.compile(r'[^0-9]')
PIS_WEIGHTS = [3, 2, 9, 8, 7, 6, 5, 4, 3, 2]


def clean_pis(pis):
    """Takes a PIS/PASEP and turns it into a string of only numbers."""
    return NONDIGIT.sub('', str(pis))

def validate_pis(pis):
    """Check whether PIS/PASEP is valid."""
    pis = clean_pis(pis)
    # all complete PIS/PASEP are 11 digits long
    if len(pis) != 11:
        return False
    return int(pis[-1]) == pis_check_digit(pis)

def pis_check_digit(pis):
    """Find check digit needed to make a PIS/PASEP valid."""
    pis = clean_pis(pis)
    if len(pis) < 10:
        raise ValueError('PIS/PASEP must be at least 10 digits: {0}'.format(pis))
    digits = [int(k) for k in pis[:11]]
    # find check digit
    cs = sum([mul(*k) for k in zip(PIS_WEIGHTS, digits)]) % 11
    return 0 if cs < 2 else 11 - cs

def pis_check_digits(pis):
    """Alias for pis_check_digit function. PIS/PASEP uses single digit."""
    return pis_check_digit(pis)

def format_pis(pis):
    """Applies typical 000.0000.000-0 formatting to PIS/PASEP."""
    fmt = '{0}.{1}.{2}-{3}'
    pis = clean_pis(pis)
    if len(pis) < 11:
        raise ValueError('Insufficient length for PIS/PASEP: {0}'.format(pis))
    return fmt.format(pis[:3], pis[3:7], pis[7:10], pis[10])

def pad_pis(pis, validate=True):
    """Takes a PIS/PASEP that had leading zeros and pads it."""
    pis = '%0.011i' % int(clean_pis(pis))
    if validate and not validate_pis(pis):
        raise ValueError('Invalid PIS/PASEP: {0}'.format(pis))
    return pis

def random_pis(formatted=True):
    """Create a random, valid PIS identifier."""
    pis = randint(1000000000, 9999999999)
    pis = str(pis) + str(pis_check_digit(pis))
    if formatted:
        return format_pis(pis)
    return pis




from distutils.core import setup

setup(
        name = 'brazilnum',
        packages = ['brazilnum'],
        version = '0.7.5',
        description = 'Validators for Brazilian CNPJ, CEI, CPF, and PIS/PASEP',
        author = 'Chris Poliquin',
        author_email = 'cpoliquin@hbs.edu',
        url = 'https://github.com/poliquin/brazilnum',
        keywords = ['brazil', 'brasil', 'cnpj', 'cei', 'cpf', 'pis', 'pasep'],
        classifiers = [
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'License :: OSI Approved :: MIT License',
            'Operating System :: OS Independent',
            'Development Status :: 4 - Beta',
            'Intended Audience :: Developers',
            'Topic :: Utilities'
            ],
        long_description = """\
Validate Brazilian Identification Numbers
-----------------------------------------

Python functions for working with CNPJ, CEI, CPF, and PIS/PASEP numbers,
which identify firms and people in Brazil.
"""
)
